UnwindReason

Enum UnwindReason 

#[non_exhaustive]
pub enum UnwindReason { Panic(Box<dyn Any + Send>), UserTrap(Box<dyn Error + Send + Sync>), LibTrap(Trap), WasmTrap { backtrace: Backtrace, pc: usize, signal_trap: Option<TrapCode>, }, }
The reason a Wasm execution is being unwound.

Shared by both trap-handler backends. WasmTrap is only produced by the OS backend (signal handler); it is never constructed in baremetal mode.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


Panic(Box<dyn Any + Send>)

A panic caused by the host



UserTrap(Box<dyn Error + Send + Sync>)

A custom error triggered by the user



LibTrap(Trap)

A Trap triggered by a wasm libcall



WasmTrap

A trap caused by the Wasm generated code

Fields

backtrace: Backtrace

Native stack backtrace at the time the trap occurred

pc: usize

Program counter in generated code where the trap occurred

signal_trap: Option<TrapCode>

Optional trap code associated with the faulting signal

Implementations

Source

impl UnwindReason

pub fn into_trap(self) -> Trap

Convert to a Trap, or resume unwinding for the Panic variant (never returns).