install

Function install 

pub fn install(store_id: StoreId) -> Result<InterruptInstallGuard, InstallError>
Install interrupt state for the given store. Note that this function may be called more than once, and correctly maintains a stack of stores for which the state is installed.