ThreadInterruptState

Struct ThreadInterruptState 

struct ThreadInterruptState {
    active_stores: Vec<StoreId>,
    current_active_store: AtomicUsize,
    current_signal_target_store: Arc<AtomicUsize>,
}
Thread-related state; only PARTS of this struct are safe to access from within the interrupt handler.

Fields

active_stores: Vec<StoreId>

We need to maintain a stack of active stores per thread, hence the vec. This should not be touched by the interrupt handler.

current_active_store: AtomicUsize

Always stores the top entry from active_stores. Needed since a vec is not safe to access from signal handlers.

current_signal_target_store: Arc<AtomicUsize>

Shared state between the thread requesting the interrupt and the thread running the store’s code. The thread requesting the interrupt writes the ID of the store it wants to interrupt to this atomic. The interrupted thread later checks this value (through its own clone of the Arc in ThreadInterruptState) against the currently running store, and traps only if they match, recording the interrupt otherwise. Note that mutexes are not safe for use within signal handlers; only atomics can be safely used.