LibCall

Enum LibCall 

#[repr(u16)]
pub enum LibCall {
Show 89 variants CeilF32 = 0, CeilF64 = 1, FloorF32 = 2, FloorF64 = 3, NearestF32 = 4, NearestF64 = 5, SqrtF32 = 6, SqrtF64 = 7, TruncF32 = 8, TruncF64 = 9, Memory32Size = 10, ImportedMemory32Size = 11, TableCopy = 12, TableInit = 13, TableFill = 14, TableSize = 15, ImportedTableSize = 16, TableGet = 17, ImportedTableGet = 18, TableSet = 19, ImportedTableSet = 20, TableGrow = 21, ImportedTableGrow = 22, FuncRef = 23, ElemDrop = 24, Memory32Copy = 25, ImportedMemory32Copy = 26, Memory32Fill = 27, ImportedMemory32Fill = 28, Memory32Init = 29, DataDrop = 30, RaiseTrap = 31, Probestack = 32, Memory32AtomicWait32 = 33, ImportedMemory32AtomicWait32 = 34, Memory32AtomicWait64 = 35, ImportedMemory32AtomicWait64 = 36, Memory32AtomicNotify = 37, ImportedMemory32AtomicNotify = 38, Throw = 39, AllocException = 40, ReadExnRef = 41, LibunwindExceptionIntoExnRef = 42, EHPersonality = 43, EHPersonality2 = 44, DebugUsize = 45, DebugStr = 46, Addsf3 = 47, Adddf3 = 48, Subsf3 = 49, Subdf3 = 50, Mulsf3 = 51, Muldf3 = 52, Divsf3 = 53, Divdf3 = 54, Negsf2 = 55, Negdf2 = 56, Extendsfdf2 = 57, Truncdfsf2 = 58, Fixsfsi = 59, Fixdfsi = 60, Fixsfdi = 61, Fixdfdi = 62, Fixunssfsi = 63, Fixunsdfsi = 64, Fixunssfdi = 65, Fixunsdfdi = 66, Floatsisf = 67, Floatsidf = 68, Floatdisf = 69, Floatdidf = 70, Floatunsisf = 71, Floatunsidf = 72, Floatundisf = 73, Floatundidf = 74, Unordsf2 = 75, Unorddf2 = 76, Eqsf2 = 77, Eqdf2 = 78, Nesf2 = 79, Nedf2 = 80, Gesf2 = 81, Gedf2 = 82, Ltsf2 = 83, Ltdf2 = 84, Lesf2 = 85, Ledf2 = 86, Gtsf2 = 87, Gtdf2 = 88,
}
The name of a runtime library routine.

This list is likely to grow over time.

Variants



CeilF32 = 0

ceil.f32



CeilF64 = 1

ceil.f64



FloorF32 = 2

floor.f32



FloorF64 = 3

floor.f64



NearestF32 = 4

nearest.f32



NearestF64 = 5

nearest.f64



SqrtF32 = 6

sqrt.f32



SqrtF64 = 7

sqrt.f64



TruncF32 = 8

trunc.f32



TruncF64 = 9

trunc.f64



Memory32Size = 10

memory.size for local functions



ImportedMemory32Size = 11

memory.size for imported functions



TableCopy = 12

table.copy



TableInit = 13

table.init



TableFill = 14

table.fill



TableSize = 15

table.size for local tables



ImportedTableSize = 16

table.size for imported tables



TableGet = 17

table.get for local tables



ImportedTableGet = 18

table.get for imported tables



TableSet = 19

table.set for local tables



ImportedTableSet = 20

table.set for imported tables



TableGrow = 21

table.grow for local tables



ImportedTableGrow = 22

table.grow for imported tables



FuncRef = 23

ref.func



ElemDrop = 24

elem.drop



Memory32Copy = 25

memory.copy for local memories



ImportedMemory32Copy = 26

memory.copy for imported memories



Memory32Fill = 27

memory.fill for local memories



ImportedMemory32Fill = 28

memory.fill for imported memories



Memory32Init = 29

memory.init



DataDrop = 30

data.drop



RaiseTrap = 31

A custom trap



Probestack = 32

probe for stack overflow. These are emitted for functions which need when the enable_probestack setting is true.



Memory32AtomicWait32 = 33

memory.atomic.wait32 for local memories



ImportedMemory32AtomicWait32 = 34

memory.atomic.wait32 for imported memories



Memory32AtomicWait64 = 35

memory.atomic.wait64 for local memories



ImportedMemory32AtomicWait64 = 36

memory.atomic.wait64 for imported memories



Memory32AtomicNotify = 37

memory.atomic.notify for local memories



ImportedMemory32AtomicNotify = 38

memory.atomic.notify for imported memories



Throw = 39

throw



AllocException = 40

allocate exception object and get an exnref for it



ReadExnRef = 41

Get the values buffer pointer out of an exnref



LibunwindExceptionIntoExnRef = 42

Given a caught native exception pointer, get the exnref and delete the exception itself



EHPersonality = 43

The personality function



EHPersonality2 = 44

The second stage of the EH personality function



DebugUsize = 45

debug_usize



DebugStr = 46

debug_str



Addsf3 = 47

__addsf3



Adddf3 = 48

__adddf3



Subsf3 = 49

__subsf3



Subdf3 = 50

__subdf3



Mulsf3 = 51

__mulsf3



Muldf3 = 52

__muldf3



Divsf3 = 53

__divsf3



Divdf3 = 54

__divdf3



Negsf2 = 55

__negsf2



Negdf2 = 56

__negdf2



Extendsfdf2 = 57

__extendsfdf2



Truncdfsf2 = 58

__truncdfsf2



Fixsfsi = 59

__fixsfsi



Fixdfsi = 60

__fixdfsi



Fixsfdi = 61

__fixsfdi



Fixdfdi = 62

__fixdfdi



Fixunssfsi = 63

__fixunssfsi



Fixunsdfsi = 64

__fixunsdfsi



Fixunssfdi = 65

__fixunssfdi



Fixunsdfdi = 66

__fixunsdfdi



Floatsisf = 67

__floatsisf



Floatsidf = 68

__floatsidf



Floatdisf = 69

__floatdisf



Floatdidf = 70

__floatdidf



Floatunsisf = 71

__floatunsisf



Floatunsidf = 72

__floatunsidf



Floatundisf = 73

__floatundisf



Floatundidf = 74

__floatundidf



Unordsf2 = 75

__unordsf2



Unorddf2 = 76

__unorddf2



Eqsf2 = 77

__eqsf2



Eqdf2 = 78

__eqdf2



Nesf2 = 79

__nesf2



Nedf2 = 80

__nedf2



Gesf2 = 81

__gesf2



Gedf2 = 82

__gedf2



Ltsf2 = 83

__ltsf2



Ltdf2 = 84

__ltdf2



Lesf2 = 85

__lesf2



Ledf2 = 86

__ledf2



Gtsf2 = 87

__gtsf2



Gtdf2 = 88

__gtdf2

Implementations



impl LibCall

pub fn to_function_name(&self) -> &str

Return the function name associated to the libcall.