WasmMmap

Struct WasmMmap 

struct WasmMmap {
    alloc: Mmap,
    size: Pages,
    vm_memory_definition: MaybeInstanceOwned<VMMemoryDefinition>,
}

Fields

alloc: Mmapsize: Pagesvm_memory_definition: MaybeInstanceOwned<VMMemoryDefinition>

The owned memory definition used by the generated code

Implementations

Source

impl WasmMmap

fn get_vm_memory_definition(&self) -> NonNull<VMMemoryDefinition>

fn size(&self) -> Pages

fn grow( &mut self, delta: Pages, conf: VMMemoryConfig, ) -> Result<Pages, MemoryError>

fn grow_at_least( &mut self, min_size: u64, conf: VMMemoryConfig, ) -> Result<(), MemoryError>

Grows the memory to at least a minimum size. If the memory is already big enough for the min size then this function does nothing

fn reset(&mut self) -> Result<(), MemoryError>

Resets the memory down to a zero size

pub fn copy(&self) -> Result<Self, MemoryError>

Copies the memory (in this case it performs a copy-on-write to save memory)