#[repr(C)]pub struct VMCallerCheckedAnyfunc {
pub func_ptr: *const VMFunctionBody,
pub type_signature_hash: VMSignatureHash,
pub vmctx: VMFunctionContext,
pub call_trampoline: VMTrampoline,
}Expand description
The VM caller-checked “anyfunc” record, for caller-side signature checking. It consists of the actual function pointer and a signature id to be checked by the caller.
Fields
func_ptr: *const VMFunctionBodyFunction body.
type_signature_hash: VMSignatureHashFunction signature id.
vmctx: VMFunctionContextFunction VMContext or host env.
call_trampoline: VMTrampolineAddress of the function call trampoline to invoke this function using a dynamic argument list.
Implementations