pub struct VMGlobal { /* private fields */ }Expand description
A Global instance
Implementations
Sourceimpl VMGlobal
impl VMGlobal
Sourcepub fn new(global_type: GlobalType) -> Self
pub fn new(global_type: GlobalType) -> Self
Create a new, zero bit-pattern initialized global from a GlobalType.
Sourcepub unsafe fn new_instance(
global_type: GlobalType,
vm_definition_location: NonNull<VMGlobalDefinition>,
) -> Self
pub unsafe fn new_instance( global_type: GlobalType, vm_definition_location: NonNull<VMGlobalDefinition>, ) -> Self
Create a new global backed by a VMGlobalDefinition stored inline in a VMContext.
Safety
vm_definition_locationmust be a valid, properly aligned location for aVMGlobalDefinition.
Sourcepub fn ty(&self) -> &GlobalType
pub fn ty(&self) -> &GlobalType
Get the type of the global.
Sourcepub fn vmglobal(&self) -> NonNull<VMGlobalDefinition>
pub fn vmglobal(&self) -> NonNull<VMGlobalDefinition>
Get a pointer to the underlying definition used by the generated code.
Sourcepub fn copy_on_write(&self) -> Self
pub fn copy_on_write(&self) -> Self
Copies this global