pub(crate) fn on_interrupted() -> boolExpand description
Called from within the signal handler to decide whether we should interrupt the currently running WASM code. This function MAY return junk results in case a signal comes in during an install or uninstall operation. However, in such cases, there is no WASM code running, and the result will be ignored by the signal handler anyway.