struct StoreInterruptState {
pthread: usize,
interrupted: bool,
thread_current_signal_target_store: Arc<AtomicUsize>,
}Expand description
All necessary data for interrupting a store running WASM code on a thread.
Fields
pthread: usizeThe pthread of the thread the store is running on, used to send the interrupt signal. Note that multiple stores may be executing WASM code within the same OS thread.
We store this as a plain integer because libc::pthread_t is a raw
pointer on some Unix targets, which would make the global DashMap
fail its Send bounds even though we only treat the value as an opaque
thread identifier.
interrupted: boolWhether this store was interrupted.
thread_current_signal_target_store: Arc<AtomicUsize>See comments in ThreadInterruptState.