pub struct SignatureRegistry { /* private fields */ }Expand description
WebAssembly requires that the caller and callee signatures in an indirect call must match. To implement this efficiently, keep a registry of all signatures, shared by all instances, so that call sites can just do an index comparison.
Implementations
Sourceimpl SignatureRegistry
impl SignatureRegistry
Sourcepub fn register(
&self,
sig: &FunctionType,
hash: SignatureHash,
) -> VMSignatureHash
pub fn register( &self, sig: &FunctionType, hash: SignatureHash, ) -> VMSignatureHash
Register a signature and return its unique hash.
Sourcepub fn lookup_signature(
&self,
sig_hash: VMSignatureHash,
) -> Option<FunctionType>
pub fn lookup_signature( &self, sig_hash: VMSignatureHash, ) -> Option<FunctionType>
Looks up a registered signature by its hash.